#!/usr/bin/env python

"""
Common handler functionality for different entities.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.data import get_address, get_uri, make_freebusy, to_part, \
                           uri_dict
from imiptools.dates import format_datetime
from imiptools.period import Period

class CommonFreebusy:

    "Common free/busy mix-in."

    def request(self):

        """
        Respond to a request by preparing a reply containing free/busy
        information for each indicated attendee.
        """

        oa = self.require_organiser_and_attendees()
        if not oa:
            return

        (organiser, organiser_attr), attendees = oa

        # Get the details for each attendee.

        responses = []
        rwrite = responses.append

        # For replies, the organiser and attendee are preserved.

        for attendee, attendee_attr in attendees.items():
            freebusy = self.store.get_freebusy(attendee)

            # Indicate the actual sender of the reply.

            if self.messenger:
                attendee_attr["SENT-BY"] = get_uri(self.messenger.sender)

            tzid = self.obj.get_tzid() or self.get_tzid()
            dtstart = self.obj.get_datetime("DTSTART")
            dtend = self.obj.get_datetime("DTEND")
            period = dtstart and dtend and Period(dtstart, dtend, tzid) or None

            rwrite(make_freebusy(freebusy, self.uid, organiser, organiser_attr, attendee, attendee_attr, period))

        # Return the reply.

        self.add_result("REPLY", [get_address(organiser)], to_part("REPLY", responses))

class Outgoing:

    "Common outgoing message handling functionality."

    def update_event_in_freebusy(self, from_organiser=True):

        "Update free/busy information when handling an object."

        freebusy = self.store.get_freebusy(self.user)

        # Use the stored event in case the reply is incomplete, as is seen
        # when Claws sends a REPLY for an object originally employing
        # recurrence information.

        obj = self.get_definitive_object(from_organiser)
        if not obj:
            return False # although this should not happen

        # If newer than any old version, discard old details from the
        # free/busy record and check for suitability.

        # Interpretation of periods can depend on the time zone.

        periods = obj.get_periods(self.get_tzid(), self.get_window_end())

        # Obtain the attendance attributes for this user, if available.

        attendees = uri_dict(self.obj.get_value_map("ATTENDEE"))
        self.update_freebusy_for_participant(freebusy, periods, attendees.get(self.user), from_organiser)

        # Remove original recurrence details replaced by additional
        # recurrences, as well as obsolete additional recurrences.

        self.remove_freebusy_for_recurrences(freebusy, self.store.get_recurrences(self.user, self.uid))
        self.store.set_freebusy(self.user, freebusy)

        if self.publisher and self.is_sharing():
            self.publisher.set_freebusy(self.user, freebusy)

        return True

    def remove_event_from_freebusy(self):

        "Remove free/busy information when handling an object."

        freebusy = self.store.get_freebusy(self.user)
        self.remove_from_freebusy(freebusy)
        self.remove_freebusy_for_recurrences(freebusy)
        self.store.set_freebusy(self.user, freebusy)

        if self.publisher and self.is_sharing():
            self.publisher.set_freebusy(self.user, freebusy)

# vim: tabstop=4 expandtab shiftwidth=4
